/*!
 *  Hack v2.020 - https://sourcefoundry.org/hack/
 *  Licenses - Fonts: Hack Open Font License + Bitstream Vera license, CSS: MIT License
 */
/* FONT PATHS
 * -------------------------- */
@font-face {
  font-family: 'Hack';
  src: url('hack/eot/hack-regular-webfont.eot?v=2.020');
  src: url('hack/eot/hack-regular-webfont.eot?#iefix&v=2.020') format('embedded-opentype'), url('hack/woff2/hack-regular-webfont.woff2?v=2.020') format('woff2'), url('hack/woff/hack-regular-webfont.woff?v=2.020') format('woff'), url('hack/web-ttf/hack-regular-webfont.ttf?v=2.020') format('truetype');
  font-weight: 400;
  font-style: normal;
}

@font-face {
  font-family: 'Hack';
  src: url('hack/eot/hack-bold-webfont.eot?v=2.020');
  src: url('hack/eot/hack-bold-webfont.eot?#iefix&v=2.020') format('embedded-opentype'), url('hack/woff2/hack-bold-webfont.woff2?v=2.020') format('woff2'), url('hack/woff/hack-bold-webfont.woff?v=2.020') format('woff'), url('hack/web-ttf/hack-bold-webfont.ttf?v=2.020') format('truetype');
  font-weight: 700;
  font-style: normal;
}

@font-face {
  font-family: 'Hack';
  src: url('hack/eot/hack-italic-webfont.eot?v=2.020');
  src: url('hack/eot/hack-italic-webfont.eot?#iefix&v=2.020') format('embedded-opentype'), url('hack/woff2/hack-italic-webfont.woff2?v=2.020') format('woff2'), url('hack/woff/hack-italic-webfont.woff?v=2.020') format('woff'), url('hack/web-ttf/hack-italic-webfont.ttf?v=2.020') format('truetype');
  font-weight: 400;
  font-style: italic;
}

@font-face {
  font-family: 'Hack';
  src: url('hack/eot/hack-bolditalic-webfont.eot?v=2.020');
  src: url('hack/eot/hack-bolditalic-webfont.eot?#iefix&v=2.020') format('embedded-opentype'), url('hack/woff2/hack-bolditalic-webfont.woff2?v=2.020') format('woff2'), url('hack/woff/hack-bolditalic-webfont.woff?v=2.020') format('woff'), url('hack/web-ttf/hack-bolditalic-webfont.ttf?v=2.020') format('truetype');
  font-weight: 700;
  font-style: italic;
}
