/* open-sans-300 - latin_latin-ext */
@font-face {
  font-family: 'Open Sans';
  font-style: normal;
  font-weight: 300;
  src: url('open-sans/open-sans-v13-latin_latin-ext-300.eot'); /* IE9 Compat Modes */
  src: local('Open Sans Light'), local('OpenSans-Light'),
       url('open-sans/open-sans-v13-latin_latin-ext-300.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('open-sans/open-sans-v13-latin_latin-ext-300.woff2') format('woff2'), /* Super Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-300.woff') format('woff'), /* Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-300.ttf') format('truetype'), /* Safari, Android, iOS */
       url('open-sans/open-sans-v13-latin_latin-ext-300.svg#OpenSans') format('svg'); /* Legacy iOS */
}

/* open-sans-300italic - latin_latin-ext */
@font-face {
  font-family: 'Open Sans';
  font-style: italic;
  font-weight: 300;
  src: url('open-sans/open-sans-v13-latin_latin-ext-300italic.eot'); /* IE9 Compat Modes */
  src: local('Open Sans Light Italic'), local('OpenSansLight-Italic'),
       url('open-sans/open-sans-v13-latin_latin-ext-300italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('open-sans/open-sans-v13-latin_latin-ext-300italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-300italic.woff') format('woff'), /* Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-300italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('open-sans/open-sans-v13-latin_latin-ext-300italic.svg#OpenSans') format('svg'); /* Legacy iOS */
}

/* open-sans-regular - latin_latin-ext */
@font-face {
  font-family: 'Open Sans';
  font-style: normal;
  font-weight: 400;
  src: url('open-sans/open-sans-v13-latin_latin-ext-regular.eot'); /* IE9 Compat Modes */
  src: local('Open Sans'), local('OpenSans'),
       url('open-sans/open-sans-v13-latin_latin-ext-regular.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('open-sans/open-sans-v13-latin_latin-ext-regular.woff2') format('woff2'), /* Super Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-regular.woff') format('woff'), /* Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-regular.ttf') format('truetype'), /* Safari, Android, iOS */
       url('open-sans/open-sans-v13-latin_latin-ext-regular.svg#OpenSans') format('svg'); /* Legacy iOS */
}

/* open-sans-italic - latin_latin-ext */
@font-face {
  font-family: 'Open Sans';
  font-style: italic;
  font-weight: 400;
  src: url('open-sans/open-sans-v13-latin_latin-ext-italic.eot'); /* IE9 Compat Modes */
  src: local('Open Sans Italic'), local('OpenSans-Italic'),
       url('open-sans/open-sans-v13-latin_latin-ext-italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('open-sans/open-sans-v13-latin_latin-ext-italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-italic.woff') format('woff'), /* Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('open-sans/open-sans-v13-latin_latin-ext-italic.svg#OpenSans') format('svg'); /* Legacy iOS */
}

/* open-sans-700 - latin_latin-ext */
@font-face {
  font-family: 'Open Sans';
  font-style: normal;
  font-weight: 700;
  src: url('open-sans/open-sans-v13-latin_latin-ext-700.eot'); /* IE9 Compat Modes */
  src: local('Open Sans Bold'), local('OpenSans-Bold'),
       url('open-sans/open-sans-v13-latin_latin-ext-700.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('open-sans/open-sans-v13-latin_latin-ext-700.woff2') format('woff2'), /* Super Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-700.woff') format('woff'), /* Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-700.ttf') format('truetype'), /* Safari, Android, iOS */
       url('open-sans/open-sans-v13-latin_latin-ext-700.svg#OpenSans') format('svg'); /* Legacy iOS */
}

/* open-sans-700italic - latin_latin-ext */
@font-face {
  font-family: 'Open Sans';
  font-style: italic;
  font-weight: 700;
  src: url('open-sans/open-sans-v13-latin_latin-ext-700italic.eot'); /* IE9 Compat Modes */
  src: local('Open Sans Bold Italic'), local('OpenSans-BoldItalic'),
       url('open-sans/open-sans-v13-latin_latin-ext-700italic.eot?#iefix') format('embedded-opentype'), /* IE6-IE8 */
       url('open-sans/open-sans-v13-latin_latin-ext-700italic.woff2') format('woff2'), /* Super Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-700italic.woff') format('woff'), /* Modern Browsers */
       url('open-sans/open-sans-v13-latin_latin-ext-700italic.ttf') format('truetype'), /* Safari, Android, iOS */
       url('open-sans/open-sans-v13-latin_latin-ext-700italic.svg#OpenSans') format('svg'); /* Legacy iOS */
}
