<%inherit file="master.html"/>
<div class="row page-header">
  <div class="col-md-6 col-md-offset-3">
    <h1>Bodhi Masher Activity</h1>
  </div>
</div>
<div class="row">
  <div class="col-md-6 col-md-offset-3" id="container">
    <img class='spinner' src='static/img/spinner.gif'>
  </div>
</div>

<script src="${request.static_url('bodhi:server/static/js/masher-status.js')}"></script>
