<%!
def inherit(context):
  if context.get('chrome'):
    return 'master.html'
  return None
%>
<%inherit file="${inherit(context)}"/>

<%namespace name="tables" file="tables.html"/>

<link rel="alternate" type="application/atom+xml" title="New Buildroot Overrides" href="${request.route_url('overrides_rss') + '?' + request.query_string}"/>

% if chrome:
<div class="row">
  <div class="col-md-12">
    <h3>Overrides <small>page #${page} of ${pages} pages
      % if page == 1:
      <a href="${request.route_url('overrides_rss') + '?' + request.query_string}">
        <span class="fa fa-rss"></span>
      </a>
      % endif
      </small>
    </h3>
    ${self.pager.render(page, pages)}
% endif
    ${tables.overrides(overrides, display_user)}
% if chrome:
    ${self.pager.render(page, pages)}
  </div>
</div>
% endif
