
.carousel {
    width: 100%;
    background: #4c4c4c;
}

.carousel-inner {
    text-align: center;
}

.carousel .item > img {
    display: inline-block;
}

.carousel-caption {
    background-color: rgba(0,0,0,0.8);
    -moz-border-radius: 15px;
    border-radius: 15px;
    padding-left: 30px;
    padding-right: 30px;
    margin: auto;
    max-width: 600px;
}

.carousel-caption > h3 {
    margin-top: -5px;
}

.example-message strong {
    font-size: 9pt;
}
.example-message img {
    width: 24px;
    height: 24px;
}
