/*!
 * Bootstrap v3.0.2 by @fat and @mdo
 * Copyright 2013 Twitter, Inc.
 * Licensed under http://www.apache.org/licenses/LICENSE-2.0
 *
 * Designed and built with all the love in the world by @mdo and @fat.
 */

.btn-default,
.btn-primary,
.btn-success,
.btn-info,
.btn-warning,
.btn-danger {
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.2);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.15), 0 1px 1px rgba(0, 0, 0, 0.075);
          box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.15), 0 1px 1px rgba(0, 0, 0, 0.075);
}

.btn-default:active,
.btn-primary:active,
.btn-success:active,
.btn-info:active,
.btn-warning:active,
.btn-danger:active,
.btn-default.active,
.btn-primary.active,
.btn-success.active,
.btn-info.active,
.btn-warning.active,
.btn-danger.active {
  -webkit-box-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
          box-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
}

.btn:active,
.btn.active {
  background-image: none;
}

.btn-default {
  text-shadow: 0 1px 0 #fff;
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#ffffff), to(#e0e0e0));
  background-image: -webkit-linear-gradient(top, #ffffff 0%, #e0e0e0 100%);
  background-image: -moz-linear-gradient(top, #ffffff 0%, #e0e0e0 100%);
  background-image: linear-gradient(to bottom, #ffffff 0%, #e0e0e0 100%);
  background-repeat: repeat-x;
  border-color: #dbdbdb;
  border-color: #ccc;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffffff', endColorstr='#ffe0e0e0', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.btn-default:hover,
.btn-default:focus {
  background-color: #e0e0e0;
  background-position: 0 -15px;
}

.btn-default:active,
.btn-default.active {
  background-color: #e0e0e0;
  border-color: #dbdbdb;
}

.btn-primary {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#428bca), to(#2d6ca2));
  background-image: -webkit-linear-gradient(top, #428bca 0%, #2d6ca2 100%);
  background-image: -moz-linear-gradient(top, #428bca 0%, #2d6ca2 100%);
  background-image: linear-gradient(to bottom, #428bca 0%, #2d6ca2 100%);
  background-repeat: repeat-x;
  border-color: #2b669a;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff428bca', endColorstr='#ff2d6ca2', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.btn-primary:hover,
.btn-primary:focus {
  background-color: #2d6ca2;
  background-position: 0 -15px;
}

.btn-primary:active,
.btn-primary.active {
  background-color: #2d6ca2;
  border-color: #2b669a;
}

.btn-success {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#79db32), to(#5cb11f));
  background-image: -webkit-linear-gradient(top, #79db32 0%, #5cb11f 100%);
  background-image: -moz-linear-gradient(top, #79db32 0%, #5cb11f 100%);
  background-image: linear-gradient(to bottom, #79db32 0%, #5cb11f 100%);
  background-repeat: repeat-x;
  border-color: #58a81e;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff79db32', endColorstr='#ff5cb11f', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.btn-success:hover,
.btn-success:focus {
  background-color: #5cb11f;
  background-position: 0 -15px;
}

.btn-success:active,
.btn-success.active {
  background-color: #5cb11f;
  border-color: #58a81e;
}

.btn-warning {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#e59728), to(#b97616));
  background-image: -webkit-linear-gradient(top, #e59728 0%, #b97616 100%);
  background-image: -moz-linear-gradient(top, #e59728 0%, #b97616 100%);
  background-image: linear-gradient(to bottom, #e59728 0%, #b97616 100%);
  background-repeat: repeat-x;
  border-color: #b07015;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffe59728', endColorstr='#ffb97616', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.btn-warning:hover,
.btn-warning:focus {
  background-color: #b97616;
  background-position: 0 -15px;
}

.btn-warning:active,
.btn-warning.active {
  background-color: #b97616;
  border-color: #b07015;
}

.btn-danger {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#db3279), to(#b11f5c));
  background-image: -webkit-linear-gradient(top, #db3279 0%, #b11f5c 100%);
  background-image: -moz-linear-gradient(top, #db3279 0%, #b11f5c 100%);
  background-image: linear-gradient(to bottom, #db3279 0%, #b11f5c 100%);
  background-repeat: repeat-x;
  border-color: #a81e58;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffdb3279', endColorstr='#ffb11f5c', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.btn-danger:hover,
.btn-danger:focus {
  background-color: #b11f5c;
  background-position: 0 -15px;
}

.btn-danger:active,
.btn-danger.active {
  background-color: #b11f5c;
  border-color: #a81e58;
}

.btn-info {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#a07cbc), to(#8255a6));
  background-image: -webkit-linear-gradient(top, #a07cbc 0%, #8255a6 100%);
  background-image: -moz-linear-gradient(top, #a07cbc 0%, #8255a6 100%);
  background-image: linear-gradient(to bottom, #a07cbc 0%, #8255a6 100%);
  background-repeat: repeat-x;
  border-color: #7d519f;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffa07cbc', endColorstr='#ff8255a6', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.btn-info:hover,
.btn-info:focus {
  background-color: #8255a6;
  background-position: 0 -15px;
}

.btn-info:active,
.btn-info.active {
  background-color: #8255a6;
  border-color: #7d519f;
}

.thumbnail,
.img-thumbnail {
  -webkit-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.075);
          box-shadow: 0 1px 2px rgba(0, 0, 0, 0.075);
}

.dropdown-menu > li > a:hover,
.dropdown-menu > li > a:focus {
  background-color: #e8e8e8;
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f5f5f5), to(#e8e8e8));
  background-image: -webkit-linear-gradient(top, #f5f5f5 0%, #e8e8e8 100%);
  background-image: -moz-linear-gradient(top, #f5f5f5 0%, #e8e8e8 100%);
  background-image: linear-gradient(to bottom, #f5f5f5 0%, #e8e8e8 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff5f5f5', endColorstr='#ffe8e8e8', GradientType=0);
}

.dropdown-menu > .active > a,
.dropdown-menu > .active > a:hover,
.dropdown-menu > .active > a:focus {
  background-color: #357ebd;
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#428bca), to(#357ebd));
  background-image: -webkit-linear-gradient(top, #428bca 0%, #357ebd 100%);
  background-image: -moz-linear-gradient(top, #428bca 0%, #357ebd 100%);
  background-image: linear-gradient(to bottom, #428bca 0%, #357ebd 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff428bca', endColorstr='#ff357ebd', GradientType=0);
}

.navbar-default {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#ffffff), to(#f8f8f8));
  background-image: -webkit-linear-gradient(top, #ffffff 0%, #f8f8f8 100%);
  background-image: -moz-linear-gradient(top, #ffffff 0%, #f8f8f8 100%);
  background-image: linear-gradient(to bottom, #ffffff 0%, #f8f8f8 100%);
  background-repeat: repeat-x;
  border-radius: 4px;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffffffff', endColorstr='#fff8f8f8', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.15), 0 1px 5px rgba(0, 0, 0, 0.075);
          box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.15), 0 1px 5px rgba(0, 0, 0, 0.075);
}

.navbar-default .navbar-nav > .active > a {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#ebebeb), to(#f3f3f3));
  background-image: -webkit-linear-gradient(top, #ebebeb 0%, #f3f3f3 100%);
  background-image: -moz-linear-gradient(top, #ebebeb 0%, #f3f3f3 100%);
  background-image: linear-gradient(to bottom, #ebebeb 0%, #f3f3f3 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffebebeb', endColorstr='#fff3f3f3', GradientType=0);
  -webkit-box-shadow: inset 0 3px 9px rgba(0, 0, 0, 0.075);
          box-shadow: inset 0 3px 9px rgba(0, 0, 0, 0.075);
}

.navbar-brand,
.navbar-nav > li > a {
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.25);
}

.navbar-inverse {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#365698), to(#294172));
  background-image: -webkit-linear-gradient(top, #365698 0%, #294172 100%);
  background-image: -moz-linear-gradient(top, #365698 0%, #294172 100%);
  background-image: linear-gradient(to bottom, #365698 0%, #294172 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff365698', endColorstr='#ff294172', GradientType=0);
  filter: progid:DXImageTransform.Microsoft.gradient(enabled=false);
}

.navbar-inverse .navbar-nav > .active > a {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#294172), to(#2c467b));
  background-image: -webkit-linear-gradient(top, #294172 0%, #2c467b 100%);
  background-image: -moz-linear-gradient(top, #294172 0%, #2c467b 100%);
  background-image: linear-gradient(to bottom, #294172 0%, #2c467b 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff294172', endColorstr='#ff2c467b', GradientType=0);
  -webkit-box-shadow: inset 0 3px 9px rgba(0, 0, 0, 0.25);
          box-shadow: inset 0 3px 9px rgba(0, 0, 0, 0.25);
}

.navbar-inverse .navbar-brand,
.navbar-inverse .navbar-nav > li > a {
  text-shadow: 0 -1px 0 rgba(0, 0, 0, 0.25);
}

.navbar-static-top,
.navbar-fixed-top,
.navbar-fixed-bottom {
  border-radius: 0;
}

.alert {
  text-shadow: 0 1px 0 rgba(255, 255, 255, 0.2);
  -webkit-box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25), 0 1px 2px rgba(0, 0, 0, 0.05);
          box-shadow: inset 0 1px 0 rgba(255, 255, 255, 0.25), 0 1px 2px rgba(0, 0, 0, 0.05);
}

.alert-success {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#dcf6ca), to(#c7f0a9));
  background-image: -webkit-linear-gradient(top, #dcf6ca 0%, #c7f0a9 100%);
  background-image: -moz-linear-gradient(top, #dcf6ca 0%, #c7f0a9 100%);
  background-image: linear-gradient(to bottom, #dcf6ca 0%, #c7f0a9 100%);
  background-repeat: repeat-x;
  border-color: #b2ea89;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffdcf6ca', endColorstr='#ffc7f0a9', GradientType=0);
}

.alert-info {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f6f2f8), to(#e3d9eb));
  background-image: -webkit-linear-gradient(top, #f6f2f8 0%, #e3d9eb 100%);
  background-image: -moz-linear-gradient(top, #f6f2f8 0%, #e3d9eb 100%);
  background-image: linear-gradient(to bottom, #f6f2f8 0%, #e3d9eb 100%);
  background-repeat: repeat-x;
  border-color: #d1bfdf;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff6f2f8', endColorstr='#ffe3d9eb', GradientType=0);
}

.alert-warning {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f8e4c7), to(#f4d4a5));
  background-image: -webkit-linear-gradient(top, #f8e4c7 0%, #f4d4a5 100%);
  background-image: -moz-linear-gradient(top, #f8e4c7 0%, #f4d4a5 100%);
  background-image: linear-gradient(to bottom, #f8e4c7 0%, #f4d4a5 100%);
  background-repeat: repeat-x;
  border-color: #f0c383;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff8e4c7', endColorstr='#fff4d4a5', GradientType=0);
}

.alert-danger {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f6cadc), to(#f0a9c7));
  background-image: -webkit-linear-gradient(top, #f6cadc 0%, #f0a9c7 100%);
  background-image: -moz-linear-gradient(top, #f6cadc 0%, #f0a9c7 100%);
  background-image: linear-gradient(to bottom, #f6cadc 0%, #f0a9c7 100%);
  background-repeat: repeat-x;
  border-color: #ea89b2;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff6cadc', endColorstr='#fff0a9c7', GradientType=0);
}

.progress {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#ebebeb), to(#f5f5f5));
  background-image: -webkit-linear-gradient(top, #ebebeb 0%, #f5f5f5 100%);
  background-image: -moz-linear-gradient(top, #ebebeb 0%, #f5f5f5 100%);
  background-image: linear-gradient(to bottom, #ebebeb 0%, #f5f5f5 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffebebeb', endColorstr='#fff5f5f5', GradientType=0);
}

.progress-bar {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#428bca), to(#3071a9));
  background-image: -webkit-linear-gradient(top, #428bca 0%, #3071a9 100%);
  background-image: -moz-linear-gradient(top, #428bca 0%, #3071a9 100%);
  background-image: linear-gradient(to bottom, #428bca 0%, #3071a9 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff428bca', endColorstr='#ff3071a9', GradientType=0);
}

.progress-bar-success {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#79db32), to(#61b921));
  background-image: -webkit-linear-gradient(top, #79db32 0%, #61b921 100%);
  background-image: -moz-linear-gradient(top, #79db32 0%, #61b921 100%);
  background-image: linear-gradient(to bottom, #79db32 0%, #61b921 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff79db32', endColorstr='#ff61b921', GradientType=0);
}

.progress-bar-info {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#a07cbc), to(#885aab));
  background-image: -webkit-linear-gradient(top, #a07cbc 0%, #885aab 100%);
  background-image: -moz-linear-gradient(top, #a07cbc 0%, #885aab 100%);
  background-image: linear-gradient(to bottom, #a07cbc 0%, #885aab 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffa07cbc', endColorstr='#ff885aab', GradientType=0);
}

.progress-bar-warning {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#e59728), to(#c27c18));
  background-image: -webkit-linear-gradient(top, #e59728 0%, #c27c18 100%);
  background-image: -moz-linear-gradient(top, #e59728 0%, #c27c18 100%);
  background-image: linear-gradient(to bottom, #e59728 0%, #c27c18 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffe59728', endColorstr='#ffc27c18', GradientType=0);
}

.progress-bar-danger {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#db3279), to(#b92161));
  background-image: -webkit-linear-gradient(top, #db3279 0%, #b92161 100%);
  background-image: -moz-linear-gradient(top, #db3279 0%, #b92161 100%);
  background-image: linear-gradient(to bottom, #db3279 0%, #b92161 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffdb3279', endColorstr='#ffb92161', GradientType=0);
}

.list-group {
  border-radius: 4px;
  -webkit-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.075);
          box-shadow: 0 1px 2px rgba(0, 0, 0, 0.075);
}

.list-group-item.active,
.list-group-item.active:hover,
.list-group-item.active:focus {
  text-shadow: 0 -1px 0 #3071a9;
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#428bca), to(#3278b3));
  background-image: -webkit-linear-gradient(top, #428bca 0%, #3278b3 100%);
  background-image: -moz-linear-gradient(top, #428bca 0%, #3278b3 100%);
  background-image: linear-gradient(to bottom, #428bca 0%, #3278b3 100%);
  background-repeat: repeat-x;
  border-color: #3278b3;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff428bca', endColorstr='#ff3278b3', GradientType=0);
}

.panel {
  -webkit-box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
          box-shadow: 0 1px 2px rgba(0, 0, 0, 0.05);
}

.panel-default > .panel-heading {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f5f5f5), to(#e8e8e8));
  background-image: -webkit-linear-gradient(top, #f5f5f5 0%, #e8e8e8 100%);
  background-image: -moz-linear-gradient(top, #f5f5f5 0%, #e8e8e8 100%);
  background-image: linear-gradient(to bottom, #f5f5f5 0%, #e8e8e8 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff5f5f5', endColorstr='#ffe8e8e8', GradientType=0);
}

.panel-primary > .panel-heading {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#428bca), to(#357ebd));
  background-image: -webkit-linear-gradient(top, #428bca 0%, #357ebd 100%);
  background-image: -moz-linear-gradient(top, #428bca 0%, #357ebd 100%);
  background-image: linear-gradient(to bottom, #428bca 0%, #357ebd 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ff428bca', endColorstr='#ff357ebd', GradientType=0);
}

.panel-success > .panel-heading {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#dcf6ca), to(#cef2b4));
  background-image: -webkit-linear-gradient(top, #dcf6ca 0%, #cef2b4 100%);
  background-image: -moz-linear-gradient(top, #dcf6ca 0%, #cef2b4 100%);
  background-image: linear-gradient(to bottom, #dcf6ca 0%, #cef2b4 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffdcf6ca', endColorstr='#ffcef2b4', GradientType=0);
}

.panel-info > .panel-heading {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f6f2f8), to(#e9e1f0));
  background-image: -webkit-linear-gradient(top, #f6f2f8 0%, #e9e1f0 100%);
  background-image: -moz-linear-gradient(top, #f6f2f8 0%, #e9e1f0 100%);
  background-image: linear-gradient(to bottom, #f6f2f8 0%, #e9e1f0 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff6f2f8', endColorstr='#ffe9e1f0', GradientType=0);
}

.panel-warning > .panel-heading {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f8e4c7), to(#f6d9b0));
  background-image: -webkit-linear-gradient(top, #f8e4c7 0%, #f6d9b0 100%);
  background-image: -moz-linear-gradient(top, #f8e4c7 0%, #f6d9b0 100%);
  background-image: linear-gradient(to bottom, #f8e4c7 0%, #f6d9b0 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff8e4c7', endColorstr='#fff6d9b0', GradientType=0);
}

.panel-danger > .panel-heading {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#f6cadc), to(#f2b4ce));
  background-image: -webkit-linear-gradient(top, #f6cadc 0%, #f2b4ce 100%);
  background-image: -moz-linear-gradient(top, #f6cadc 0%, #f2b4ce 100%);
  background-image: linear-gradient(to bottom, #f6cadc 0%, #f2b4ce 100%);
  background-repeat: repeat-x;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#fff6cadc', endColorstr='#fff2b4ce', GradientType=0);
}

.well {
  background-image: -webkit-gradient(linear, left 0%, left 100%, from(#e8e8e8), to(#f5f5f5));
  background-image: -webkit-linear-gradient(top, #e8e8e8 0%, #f5f5f5 100%);
  background-image: -moz-linear-gradient(top, #e8e8e8 0%, #f5f5f5 100%);
  background-image: linear-gradient(to bottom, #e8e8e8 0%, #f5f5f5 100%);
  background-repeat: repeat-x;
  border-color: #dcdcdc;
  filter: progid:DXImageTransform.Microsoft.gradient(startColorstr='#ffe8e8e8', endColorstr='#fff5f5f5', GradientType=0);
  -webkit-box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.05), 0 1px 0 rgba(255, 255, 255, 0.1);
          box-shadow: inset 0 1px 3px rgba(0, 0, 0, 0.05), 0 1px 0 rgba(255, 255, 255, 0.1);
}